@@ -70,12 +70,71 @@ static PyMethodDef functions[] = {
   {NULL, NULL, 0, NULL}
 };
 
-PyMODINIT_FUNC init_spglib(void)
-{
-  Py_InitModule3("_spglib", functions, "C-extension for spglib\n\n...\n");
-  return;
+struct module_state {
+    PyObject *error;
+};
+
+#if PY_MAJOR_VERSION >= 3
+#define GETSTATE(m) ((struct module_state*)PyModule_GetState(m))
+#else
+#define GETSTATE(m) (&_state)
+static struct module_state _state;
+#endif
+
+static PyObject *
+error_out(PyObject *m) {
+    struct module_state *st = GETSTATE(m);
+    PyErr_SetString(st->error, "something bad happened");
+    return NULL;
 }
 
+#if PY_MAJOR_VERSION >= 3
+
+static int _spglib_traverse(PyObject *m, visitproc visit, void *arg) {
+    Py_VISIT(GETSTATE(m)->error);
+    return 0;
+}
+
+static int _spglib_clear(PyObject *m) {
+    Py_CLEAR(GETSTATE(m)->error);
+    return 0;
+}
+
+
+static struct PyModuleDef moduledef = {
+        PyModuleDef_HEAD_INIT,
+        "_spglib",
+        NULL,
+        sizeof(struct module_state),
+        functions,
+        NULL,
+        _spglib_traverse,
+        _spglib_clear,
+        NULL
+};
+
+
+
+#define INITERROR return NULL
+
+PyMODINIT_FUNC PyInit__spglib(void)
+
+#else
+#define INITERROR return
+
+void init_spglib(void)
+#endif
+{
+#if PY_MAJOR_VERSION >= 3
+    PyObject *module = PyModule_Create(&moduledef);
+    return module;
+#else
+    (void) Py_InitModule("_spglib", functions);
+#endif
+
+}
+
+
 static PyObject * get_dataset(PyObject *self, PyObject *args)
 {
   int i, j, k;
@@ -85,7 +144,7 @@ static PyObject * get_dataset(PyObject *self, PyObject *args)
   PyArrayObject* position;
   PyArrayObject* atom_type;
   PyObject* array, *vec, *mat, *rot, *trans, *wyckoffs, *equiv_atoms;
-  
+
   if (!PyArg_ParseTuple(args, "OOOdd",
 			&lattice,
 			&position,
@@ -110,9 +169,9 @@ static PyObject * get_dataset(PyObject *self, PyObject *args)
   array = PyList_New(9);
 
   /* Space group number, international symbol, hall symbol */
-  PyList_SetItem(array, 0, PyInt_FromLong((long) dataset->spacegroup_number));
-  PyList_SetItem(array, 1, PyString_FromString(dataset->international_symbol));
-  PyList_SetItem(array, 2, PyString_FromString(dataset->hall_symbol));
+  PyList_SetItem(array, 0, PyLong_FromLong((long) dataset->spacegroup_number));
+  PyList_SetItem(array, 1, PyUnicode_FromString(dataset->international_symbol));
+  PyList_SetItem(array, 2, PyUnicode_FromString(dataset->hall_symbol));
 
   /* Transformation matrix */
   mat = PyList_New(3);
@@ -139,7 +198,7 @@ static PyObject * get_dataset(PyObject *self, PyObject *args)
     for (j = 0; j < 3; j++) {
       vec = PyList_New(3);
       for (k = 0; k < 3; k++) {
-	PyList_SetItem(vec, k, PyInt_FromLong((long) dataset->rotations[i][j][k]));
+	PyList_SetItem(vec, k, PyLong_FromLong((long) dataset->rotations[i][j][k]));
       }
       PyList_SetItem(mat, j, vec);
     }
@@ -162,8 +221,8 @@ static PyObject * get_dataset(PyObject *self, PyObject *args)
   wyckoffs = PyList_New(dataset->n_atoms);
   equiv_atoms = PyList_New(dataset->n_atoms);
   for (i = 0; i < dataset->n_atoms; i++) {
-    PyList_SetItem(wyckoffs, i, PyInt_FromLong((long) dataset->wyckoffs[i]));
-    PyList_SetItem(equiv_atoms, i, PyInt_FromLong((long) dataset->equivalent_atoms[i]));
+    PyList_SetItem(wyckoffs, i, PyLong_FromLong((long) dataset->wyckoffs[i]));
+    PyList_SetItem(equiv_atoms, i, PyLong_FromLong((long) dataset->equivalent_atoms[i]));
   }
   PyList_SetItem(array, 7, wyckoffs);
   PyList_SetItem(array, 8, equiv_atoms);
@@ -207,7 +266,7 @@ static PyObject * get_spacegroup(PyObject *self, PyObject *args)
   spg_symbol[i + 1] = 0;
   sprintf(symbol_with_number, "%s (%d)", spg_symbol, num_spg);
 
-  return PyString_FromString(symbol_with_number);
+  return PyUnicode_FromString(symbol_with_number);
 }
 
 static PyObject * get_pointgroup(PyObject *self, PyObject *args)
@@ -231,14 +290,14 @@ static PyObject * get_pointgroup(PyObject *self, PyObject *args)
   for (i = 0; i < 3; i++) {
     vec = PyList_New(3);
     for (j = 0; j < 3; j++) {
-      PyList_SetItem(vec, j, PyInt_FromLong((long)trans_mat[i][j]));
+      PyList_SetItem(vec, j, PyLong_FromLong((long)trans_mat[i][j]));
     }
     PyList_SetItem(mat, i, vec);
   }
 
   array = PyList_New(3);
-  PyList_SetItem(array, 0, PyString_FromString(symbol));
-  PyList_SetItem(array, 1, PyInt_FromLong((long) ptg_num));
+  PyList_SetItem(array, 0, PyUnicode_FromString(symbol));
+  PyList_SetItem(array, 1, PyLong_FromLong((long) ptg_num));
   PyList_SetItem(array, 2, mat);
 
   return array;
@@ -272,7 +331,7 @@ static PyObject * refine_cell(PyObject *self, PyObject *args)
 				       symprec,
 				       angle_tolerance);
 
-  return PyInt_FromLong((long) num_atom_brv);
+  return PyLong_FromLong((long) num_atom_brv);
 }
 
 
@@ -303,7 +362,7 @@ static PyObject * find_primitive(PyObject *self, PyObject *args)
 					   symprec,
 					   angle_tolerance);
 
-  return PyInt_FromLong((long) num_atom_prim);
+  return PyLong_FromLong((long) num_atom_prim);
 }
 
 static PyObject * get_symmetry(PyObject *self, PyObject *args)
@@ -332,7 +391,7 @@ static PyObject * get_symmetry(PyObject *self, PyObject *args)
   int (*rot)[3][3] = (int(*)[3][3])rotation->data;
   double (*trans)[3] = (double(*)[3])translation->data;
   const int num_sym_from_array_size = rotation->dimensions[0];
-  
+
   /* num_sym has to be larger than num_sym_from_array_size. */
   const int num_sym = spgat_get_symmetry(rot,
 					 trans,
@@ -343,7 +402,7 @@ static PyObject * get_symmetry(PyObject *self, PyObject *args)
 					 num_atom,
 					 symprec,
 					 angle_tolerance);
-  return PyInt_FromLong((long) num_sym);
+  return PyLong_FromLong((long) num_sym);
 }
 
 static PyObject * get_symmetry_with_collinear_spin(PyObject *self,
@@ -378,7 +437,7 @@ static PyObject * get_symmetry_with_collinear_spin(PyObject *self,
   const int num_sym_from_array_size = rotation->dimensions[0];
 
   /* num_sym has to be larger than num_sym_from_array_size. */
-  const int num_sym = 
+  const int num_sym =
     spgat_get_symmetry_with_collinear_spin(rot,
 					   trans,
 					   num_sym_from_array_size,
@@ -389,7 +448,7 @@ static PyObject * get_symmetry_with_collinear_spin(PyObject *self,
 					   num_atom,
 					   symprec,
 					   angle_tolerance);
-  return PyInt_FromLong((long) num_sym);
+  return PyLong_FromLong((long) num_sym);
 }
 
 static PyObject * get_ir_reciprocal_mesh(PyObject *self, PyObject *args)
@@ -435,8 +494,8 @@ static PyObject * get_ir_reciprocal_mesh(PyObject *self, PyObject *args)
 						types,
 						num_atom,
 						symprec);
-  
-  return PyInt_FromLong((long) num_ir);
+
+  return PyLong_FromLong((long) num_ir);
 }
 
 static PyObject * get_stabilized_reciprocal_mesh(PyObject *self, PyObject *args)
@@ -477,8 +536,8 @@ static PyObject * get_stabilized_reciprocal_mesh(PyObject *self, PyObject *args)
 							rot,
 							num_q,
 							q);
-  
-  return PyInt_FromLong((long) num_ir);
+
+  return PyLong_FromLong((long) num_ir);
 }
 
 static PyObject * get_grid_points_by_rotations(PyObject *self, PyObject *args)
@@ -503,7 +562,7 @@ static PyObject * get_grid_points_by_rotations(PyObject *self, PyObject *args)
   const int num_rot = rot_reciprocal_py->dimensions[0];
   const int* mesh = (int*)mesh_py->data;
   const int* is_shift = (int*)is_shift_py->data;
-  
+
   spg_get_grid_points_by_rotations(rot_grid_points,
 				   address_orig,
 				   num_rot,
@@ -538,7 +597,7 @@ static PyObject * get_BZ_grid_points_by_rotations(PyObject *self, PyObject *args
   const int* mesh = (int*)mesh_py->data;
   const int* is_shift = (int*)is_shift_py->data;
   const int* bz_map = (int*)bz_map_py->data;
-  
+
   spg_get_BZ_grid_points_by_rotations(rot_grid_points,
 				      address_orig,
 				      num_rot,
@@ -575,7 +634,7 @@ static PyObject * relocate_BZ_grid_address(PyObject *self, PyObject *args)
   SPGCONST double (*reciprocal_lattice)[3]  =
     (double(*)[3])reciprocal_lattice_py->data;
   int num_ir_gp;
-  
+
   num_ir_gp = spg_relocate_BZ_grid_address(bz_grid_address,
 					   bz_map,
 					   grid_address,
@@ -583,7 +642,7 @@ static PyObject * relocate_BZ_grid_address(PyObject *self, PyObject *args)
 					   reciprocal_lattice,
 					   is_shift);
 
-  return PyInt_FromLong((long) num_ir_gp);
+  return PyLong_FromLong((long) num_ir_gp);
 }
 
 static PyObject * get_triplets_reciprocal_mesh_at_q(PyObject *self, PyObject *args)
@@ -613,7 +672,7 @@ static PyObject * get_triplets_reciprocal_mesh_at_q(PyObject *self, PyObject *ar
   const int* mesh_int = (int*)mesh->data;
   SPGCONST int (*rot)[3][3] = (int(*)[3][3])rotations->data;
   const int num_rot = rotations->dimensions[0];
-  const int num_ir = 
+  const int num_ir =
     spg_get_triplets_reciprocal_mesh_at_q(map_triplets_int,
 					  map_q_int,
 					  grid_address,
@@ -623,7 +682,7 @@ static PyObject * get_triplets_reciprocal_mesh_at_q(PyObject *self, PyObject *ar
 					  num_rot,
 					  rot);
 
-  return PyInt_FromLong((long) num_ir);
+  return PyLong_FromLong((long) num_ir);
 }
 
 
@@ -652,7 +711,7 @@ static PyObject * get_BZ_triplets_at_q(PyObject *self, PyObject *args)
   const int num_map_triplets = (int)map_triplets_py->dimensions[0];
   const int *mesh = (int*)mesh_py->data;
   int num_ir;
-  
+
   num_ir = spg_get_BZ_triplets_at_q(triplets,
 				    grid_point,
 				    bz_grid_address,
@@ -661,7 +720,7 @@ static PyObject * get_BZ_triplets_at_q(PyObject *self, PyObject *args)
 				    num_map_triplets,
 				    mesh);
 
-  return PyInt_FromLong((long) num_ir);
+  return PyLong_FromLong((long) num_ir);
 }
 
 static PyObject *get_neighboring_grid_points(PyObject *self, PyObject *args)
@@ -689,7 +748,7 @@ static PyObject *get_neighboring_grid_points(PyObject *self, PyObject *args)
   const int *mesh = (int*)mesh_py->data;
   SPGCONST int (*bz_grid_address)[3] = (int(*)[3])bz_grid_address_py->data;
   const int *bz_map = (int*)bz_map_py->data;
-  
+
   spg_get_neighboring_grid_points(relative_grid_points,
 				  grid_point,
 				  relative_grid_address,
@@ -705,7 +764,7 @@ get_tetrahedra_relative_grid_address(PyObject *self, PyObject *args)
 {
   PyArrayObject* relative_grid_address_py;
   PyArrayObject* reciprocal_lattice_py;
-  
+
   if (!PyArg_ParseTuple(args, "OO",
 			&relative_grid_address_py,
 			&reciprocal_lattice_py)) {
@@ -727,7 +786,7 @@ static PyObject *
 get_all_tetrahedra_relative_grid_address(PyObject *self, PyObject *args)
 {
   PyArrayObject* relative_grid_address_py;
-  
+
   if (!PyArg_ParseTuple(args, "O",
 			&relative_grid_address_py)) {
     return NULL;
@@ -756,7 +815,7 @@ get_tetrahedra_integration_weight(PyObject *self, PyObject *args)
 
   SPGCONST double (*tetrahedra_omegas)[4] =
     (double(*)[4])tetrahedra_omegas_py->data;
-  
+
   double iw = spg_get_tetrahedra_integration_weight(omega,
 						    tetrahedra_omegas,
 						    function);
@@ -784,7 +843,7 @@ get_tetrahedra_integration_weight_at_omegas(PyObject *self, PyObject *args)
   const int num_omegas = (int)omegas_py->dimensions[0];
   SPGCONST double (*tetrahedra_omegas)[4] =
     (double(*)[4])tetrahedra_omegas_py->data;
-  
+
   spg_get_tetrahedra_integration_weight_at_omegas(iw,
 						  num_omegas,
 						  omegas,
