@@ -11,8 +11,7 @@ units are detected. An ArrayWithUnit is also implemented, which is a subclass
 of numpy's ndarray with similar unit features.
 """
 
-from six.moves import filter
-from six.moves import zip
+from six.moves import filter, zip
 
 __author__ = "Shyue Ping Ong, Matteo Giantomassi"
 __copyright__ = "Copyright 2011, The Materials Project"
@@ -79,9 +78,19 @@ BASE_UNITS = {
     },
     "intensity": {
         "cd": 1
-    }
+    },
+    "memory": {
+        "byte": 1,
+        "Kb": 1024,
+        "Mb": 1024**2,
+        "Gb": 1024**3,
+        "Tb": 1024**4,
+    },
 }
 
+# Accept kb, mb, gb ... as well.
+BASE_UNITS["memory"].update({k.lower(): v for k, v in BASE_UNITS["memory"].items()})
+
 
 #This current list are supported derived units defined in terms of powers of
 #SI base units and constants.
@@ -302,13 +311,36 @@ class FloatWithUnit(float):
     >>> a = Energy(1.1, "Ha")
     >>> b = Energy(3, "eV")
     >>> c = a + b
-    >>> print c
+    >>> print(c)
     1.2102479761938871 Ha
     >>> c.to("eV")
     32.932522246000005 eV
     """
     Error = UnitError
 
+    @classmethod
+    def from_string(cls, s):
+        """
+        Initialize a FloatWithUnit from a string. Example Memory.from_string("1. Mb")
+        """
+        # Extract num and unit string. 
+        s = s.strip()
+        for i, char in enumerate(s):
+            if char.isalpha() or char.isspace():
+                break
+        else:
+            raise Exception("Unit is missing in string %s" % s)
+        num, unit = float(s[:i]), s[i:]
+
+        # Find unit type (set it to None if it cannot be detected)
+        for unit_type, d in BASE_UNITS.items():
+            if unit in d:
+                break
+        else:
+            unit_type = None
+
+        return cls(num, unit, unit_type=unit_type)
+
     def __new__(cls, val, unit, unit_type=None):
         new = float.__new__(cls, val)
         new._unit = Unit(unit)
@@ -476,7 +508,7 @@ class ArrayWithUnit(np.ndarray):
     >>> a = EnergyArray([1, 2], "Ha")
     >>> b = EnergyArray([1, 2], "eV")
     >>> c = a + b
-    >>> print c
+    >>> print(c)
     [ 1.03674933  2.07349865] Ha
     >>> c.to("eV")
     array([ 28.21138386,  56.42276772]) eV
@@ -644,6 +676,17 @@ class ArrayWithUnit(np.ndarray):
         return "\n".join(str(self.to(unit)) for unit in self.supported_units)
 
 
+def _my_partial(func, *args, **kwargs):
+    """
+    Partial returns a partial object and therefore we cannot inherit class methods defined in FloatWithUnit. 
+    This function calls partial and patches the new class before returning.
+    """
+    newobj = partial(func, *args, **kwargs)
+    # monkey patch
+    newobj.from_string = FloatWithUnit.from_string
+    return newobj
+
+
 Energy = partial(FloatWithUnit, unit_type="energy")
 """
 A float with an energy unit.
@@ -709,6 +752,17 @@ Args:
 ChargeArray = partial(ArrayWithUnit, unit_type="charge")
 
 
+Memory = _my_partial(FloatWithUnit, unit_type="memory")
+"""
+A float with a memory unit.
+
+Args:
+    val (float): Value
+    unit (Unit): E.g., Kb, Mb, Gb, Tb. Must be valid unit or UnitError
+        is raised.
+"""
+
+
 def obj_with_unit(obj, unit):
     """
     Returns a `FloatWithUnit` instance if obj is scalar, a dictionary of
