@@ -10,16 +10,13 @@ import os
 import abc
 import collections
 import json
-import warnings
 import six
 import numpy as np
 
 from monty.string import list_strings, is_string
 from monty.itertools import iterator_from_slice
 from monty.io import FileLock
-# TODO: Use Namespace (requires new monty release)
-from monty.collections import AttrDict, Namespace #NotOverwritableDict
-#Namespace = NotOverwritableDict
+from monty.collections import AttrDict, Namespace 
 from pymatgen.core.periodic_table import PeriodicTable #, Element
 
 
@@ -37,6 +34,7 @@ _PTABLE = PeriodicTable()
 
 # Tools and helper functions.
 
+
 def straceback():
     """Returns a string with the traceback."""
     import sys
@@ -114,10 +112,7 @@ class Pseudo(six.with_metaclass(abc.ABCMeta, object)):
             * Pseudo object.
             * string defining a valid path.
         """
-        if isinstance(obj, cls):
-            return obj
-        else:
-            return cls.from_file(obj)
+        return obj if isinstance(obj, cls) else cls.from_file(obj)
 
     @staticmethod
     def from_file(filename):
@@ -233,6 +228,17 @@ class Pseudo(six.with_metaclass(abc.ABCMeta, object)):
     #def generation_mode
     #    """scalar scalar-relativistic, relativistic."""
 
+    def as_dict(self, **kwargs):
+        return dict(
+            name=self.name,
+            type=self.type,
+            symbol=self.symbol,
+            Z=self.Z,
+            Z_val=self.Z_val,
+            l_max=self.l_max,
+            #nlcc_radius=self.nlcc_radius,
+        )
+
     @property
     def has_dojo_report(self):
         """True if self contains the DOJO_REPORT section."""
@@ -538,7 +544,6 @@ def _dict_from_lines(lines, key_nums, sep=None):
 
         if len(values) != len(keys):
             msg = "line: %s\n len(keys) != len(value)\nkeys: %s\n values:  %s" % (line, keys, values)
-            #warnings.warn(msg)
             raise ValueError(msg)
 
         kwargs.update(zip(keys, values))
@@ -1600,6 +1605,17 @@ class PseudoTable(collections.Sequence):
         zlist.sort()
         return zlist
 
+    def as_dict(self, **kwargs):
+        d = {}
+        for p in self:
+            k, count = p.name, 1
+            # Handle multiple-pseudos with the same name!
+            while k not in d:
+                k += k + "#" + str(count)
+                count += 1
+            d.update({k, p.as_dict()})
+        return d
+
     def is_complete(self, zmax=118):
         """
         True if table is complete i.e. all elements with Z < zmax
@@ -1701,7 +1717,7 @@ class PseudoTable(collections.Sequence):
             attrs.append((i, a))
 
         # Sort attrs, and build new table with sorted pseudos.
-        attrs = sorted(attrs, key=lambda t:t[1], reverse=reverse)
+        attrs = sorted(attrs, key=lambda t: t[1], reverse=reverse)
         return PseudoTable([self[a[0]] for a in attrs])
 
     def select(self, condition):
