@@ -33,9 +33,10 @@ from six.moves import map, zip
 import numpy as np
 
 from monty.io import zopen, reverse_readfile
+from monty.json import jsanitize
 
-from pymatgen.util.io_utils import clean_lines, micro_pyawk, \
-    clean_json
+
+from pymatgen.util.io_utils import clean_lines, micro_pyawk
 from pymatgen.core.structure import Structure
 from pymatgen.core.units import unitized
 from pymatgen.core.composition import Composition
@@ -133,6 +134,21 @@ def _parse_from_incar(filename, key):
     return None
 
 
+def _vasprun_float(f):
+    """
+    Large numbers are often represented as ********* in the vasprun.
+    This function parses these values as np.nan
+    """
+    try:
+        return float(f)
+    except ValueError as e:
+        f = f.strip()
+        if f == '*' * len(f):
+            warnings.warn('Float overflow (*******) encountered in vasprun')
+            return np.nan
+        raise e
+
+
 class Vasprun(PMGSONable):
     """
     Vastly improved cElementTree-based parser for vasprun.xml files. Uses
@@ -337,7 +353,8 @@ class Vasprun(PMGSONable):
                     self.tdos, self.idos, self.pdos = self._parse_dos(elem)
                     self.efermi = self.tdos.efermi
                     self.dos_has_errors = False
-                except Exception:
+                except Exception as ex:
+                    print(ex)
                     self.dos_has_errors = True
             elif parse_eigen and tag == "eigenvalues":
                 self.eigenvalues = self._parse_eigen(elem)
@@ -753,7 +770,7 @@ class Vasprun(PMGSONable):
 
         vout['epsilon_static'] = self.epsilon_static
         d['output'] = vout
-        return clean_json(d, strict=True)
+        return jsanitize(d, strict=True)
 
     def _parse_params(self, elem):
         params = {}
@@ -821,7 +838,7 @@ class Vasprun(PMGSONable):
                  for i in elem.find("energy").findall("i")}
         esteps = []
         for scstep in elem.findall("scstep"):
-            d = {i.attrib["name"]: float(i.text)
+            d = {i.attrib["name"]: _vasprun_float(i.text)
                  for i in scstep.find("energy").findall("i")}
             esteps.append(d)
         s = self._parse_structure(elem.find("structure"))
@@ -846,16 +863,18 @@ class Vasprun(PMGSONable):
             idensities[spin] = data[:, 2]
 
         pdoss = []
-        for s in elem.find("partial").find("array").find("set").findall("set"):
-            pdos = defaultdict(dict)
-            for ss in s.findall("set"):
-                spin = Spin.up if ss.attrib["comment"] == "spin 1" else \
-                    Spin.down
-                data = np.array(_parse_varray(ss))
-                nrow, ncol = data.shape
-                for j in range(1, ncol):
-                    pdos[Orbital.from_vasp_index(j - 1)][spin] = data[:, j]
-            pdoss.append(pdos)
+        partial = elem.find("partial")
+        if partial is not None:
+            for s in partial.find("array").find("set").findall("set"):
+                pdos = defaultdict(dict)
+                for ss in s.findall("set"):
+                    spin = Spin.up if ss.attrib["comment"] == "spin 1" else \
+                        Spin.down
+                    data = np.array(_parse_varray(ss))
+                    nrow, ncol = data.shape
+                    for j in range(1, ncol):
+                        pdos[Orbital.from_vasp_index(j - 1)][spin] = data[:, j]
+                pdoss.append(pdos)
         elem.clear()
         return Dos(efermi, energies, tdensities), \
                Dos(efermi, energies, idensities), pdoss
@@ -1005,6 +1024,7 @@ class Outcar(PMGSONable):
                 break
 
         # data from beginning of OUTCAR
+        run_stats['cores'] = 0
         with zopen(filename, "rt") as f:
             for line in f:
                 if "running" in line:
@@ -1286,13 +1306,13 @@ class Outcar(PMGSONable):
             structure at the last ionic step is [5]["2s"][-1]
         """
 
-        natom = len(self.charge)
-        cl = [defaultdict(list) for i in range(natom)]
-
         with zopen(self.filename, "rt") as foutcar:
             line = foutcar.readline()
             while line != "":
                 line = foutcar.readline()
+                if "NIONS =" in line:
+                    natom = int(line.split("NIONS =")[1])
+                    cl = [defaultdict(list) for i in range(natom)]
                 if "the core state eigen" in line:
                     for iat in range(natom):
                         line = foutcar.readline()
@@ -1998,11 +2018,11 @@ class Xdatcar(object):
                 if preamble is None:
                     preamble = [l]
                 elif not preamble_done:
-                    if l == "":
+                    if l == "" or "Direct configuration=" in l:
                         preamble_done = True
                     else:
                         preamble.append(l)
-                elif l == "":
+                elif l == "" or "Direct configuration=" in l:
                     p = Poscar.from_string("\n".join(preamble +
                                                      ["Direct"] + coords_str))
                     structures.append(p.structure)
